# Copyright 2023 Luminary Cloud, Inc. All Rights Reserved.
import logging
from os import PathLike
from typing import Optional, Union

from google.protobuf import json_format

from .._proto.fvm.param_pb2 import Param
from .._proto.simulation.simulation_pb2 import (
    Simulation,
    SimulationOptions,
    CreateSimulationRequest,
)
from .._client import Client

logger = logging.getLogger(__name__)


def create_simulation(
    client: Client,
    *,
    project_id: str = "",
    mesh_id: str = "",
    name: str = "",
    simulation_params: Optional[Param] = None,
    simulation_params_json_path: Optional[Union[PathLike, str]] = None,
    batch_processing: bool = False,
) -> Simulation:
    try:
        if simulation_params is None:
            if simulation_params_json_path is None:
                raise ValueError(
                    "Neither simulation_params nor simulation_params_json_path were set."
                )
            logger.info(f"Reading simulation params from file: {simulation_params_json_path}")
            with open(simulation_params_json_path) as fp:
                params_json = fp.read()
            simulation_params = json_format.Parse(params_json, Param())
    except:
        logger.exception("Missing simulation params.")
        raise

    logger.debug(
        f"Calling gRPC CreateSimulation with project ID {project_id} and mesh ID {mesh_id}."
    )
    response = client.CreateSimulation(
        CreateSimulationRequest(
            project_id=project_id,
            mesh_id=mesh_id,
            name=name,
            simulation_params=simulation_params,
            simulation_options=SimulationOptions(batch_processing=batch_processing),
        )
    )
    return response.simulation
